@@ -268,5 +268,35 @@ def test3d_8n_ub_eps():
     
     assert np.array_equal(idx, exp_idx)
     assert np.allclose(dist, exp_dist)
+
+def test3d_large_query():
+    # Target idxs: 7, 93, 45
+    query_pts = np.array([[  787014.438,  -340616.906,  6313018.],
+                          [751763.125, -59925.969, 6326205.5],
+                          [769957.188, -202418.125, 6321069.5]])
+
+    # Repeat the same points multiple times to get 60000 query points
+    n = 20000
+    query_pts = np.repeat(query_pts, n, axis=0)
+
+    kdtree = KDTree(data_pts_real)
+    dist, idx = kdtree.query(query_pts, sqr_dists=True)
+
+    epsilon = 1e-5
+    assert np.all(idx[:n] == 7)
+    assert np.all(idx[n:2*n] == 93)
+    assert np.all(idx[2*n:] == 45)
+    assert np.all(dist[:n] == 0)
+    assert np.all(abs(dist[n:2*n] - 3.) < epsilon * dist[n:2*n])
+    assert np.all(abs(dist[2*n:] - 20001.) < epsilon * dist[2*n:])
+
+def test_scipy_comp():
     
+    query_pts = np.array([[  787014.438,  -340616.906,  6313018.],
+                          [751763.125, -59925.969, 6326205.5],
+                          [769957.188, -202418.125, 6321069.5]])
+                          
+    kdtree = KDTree(data_pts_real)
+    assert id(kdtree.data) == id(kdtree.data_pts)
+
 
